/******************************************************************************
 Symbols.h

		Symbol data manager interface for the SignWriter symbol editor
		
		Copyright  1995 Center for Sutton Movement Writing. All rights reserved.

 ******************************************************************************/
 
#pragma once



#include "LongCoordinates.h"



		// Symbol types
		
typedef enum {face, hand, movement, shoulder, contact, other} SymbolType;



		// Image classes
		
typedef enum {class1, class2, class3A, class3B, class4, class5} ImageClass;



	   // Status
	  
Boolean  SymEmpty (short symCode);
			
			 
			 
	  // Sym count
	  
void  GetSymbols (short *numSyms);			 				  
	 
	 
	 
	  // Sym table	

void  InsertSym (short symCode);

void  DeleteSym (short symCode);



	  // Initialization
	  
void  NewSymbol (short symCode);

void  ISymbol (short symCode);

void  DisposSymbol (short symCode);



	  // Symbol type
	  
void  GetSymType (short symCode,
				  SymbolType *symType);
				  
void  SetSymType (short symCode,
				  SymbolType symType);



	  // Symbol variations
	  
void  GetSymVars (short symCode,
				  short *numVars);
				  
void  SetSymVars (short symCode,
				  short numVars);

	
				 
	  // Fill variations
	  
void  GetFillVars (short symCode, short symVar,
				   short *numFills);
				   
void  SetFillVars (short symCode, short symVar,
				   short numFills);



	  // Image class
	  
void  GetClass (short symCode, short symVar, short fillVar,
				ImageClass *imgClass);
				 
void  SetClass (short symCode, short symVar, short fillVar,
				ImageClass imgClass);



	  // Image rotations
	  
void  GetRotations (short symCode, short symVar, short fillVar,
				    short *numRots);
				 
void  SetRotations (short symCode, short symVar, short fillVar,
				    short numRots);
				   
				   

	  // Image flops
	  
void  GetFlops (short symCode, short symVar, short fillVar,
				short *numFlops);
				 
void  SetFlops (short symCode, short symVar, short fillVar,
				short numFlops);
					
				   
			
	  // Images
	  	   
void  GetImages (ImageClass imgClass, short *numImgs);

		
void  GetImage (short symCode, short symVar, short fillVar, short image, 
				BitMap *img, LongPt *frame, LongPt *center);
				
void  SetImage (short symCode, short symVar, short fillVar, short image,
				BitMap *img, LongPt *frame, LongPt *center);

void  DrawImage (short symCode, short symVar, short fillVar, short image, 
		 		 BitMap *img, LongPt *imgPos);	



	  // Symbol drawing
		 		 			
void  GetCoords (short sym, short symVar, short fillVar, short rot, short flop, 
				 LongPt *frame, LongPt *center, Boolean thumbNail);
		 		   
void  DrawSymbol (short sym, short symVar, short fillVar, short rot, short flop, 
				  BitMap *img, LongPt *imgPos, Boolean thumbNail);

			
							 
	  // Symbol editing				 				  

void  GetSymbolRec (short symCode,
				 	void **symbolRec);
				  
void  SetSymbolRec (short symCode,
				 	void *symbolRec);			

void  CopySymbolRec (void *symbolRec,
				  	 void **copyRec);

// no ClearSymbolRec!
				  	 			  	 		  
void  DisposSymbolRec (void *symbolRec);
			
			
				 
	  // Sym variation editing				 				  

void  GetSymVarRec (short symCode, short symVar,
				 	void **symVarRec);
				  
void  SetSymVarRec (short symCode, short symVar,
				 	void *symVarRec);			

void  CopySymVarRec (void *symVarRec,
				 	 void **copyRec);
				 	 
void  ClearSymVarRec (void **symVarRec);				 	
			  
void  DisposSymVarRec (void *symVarRec);				 	
				 	
				 	
				 	
	  // Fill variation editing				 				  
	 	
void  GetFillVarRec (short symCode, short symVar, short fillVar,
				 	 void **fillVarRec);
				  
void  SetFillVarRec (short symCode, short symVar, short fillVar,
				 	 void *fillVarRec);			

void  CopyFillVarRec (void *fillVarRec,
				 	  void **copyRec);

void  ClearFillVarRec (void **fillVarRec);
				  
void  DisposFillVarRec (void *fillVarRec);



 	  // Image editing				 				  

void  GetImageRec (short symCode, short symVar, short fillVar, short image,
				   void **imageRec);
				  
void  SetImageRec (short symCode, short symVar, short fillVar, short image,
				   void *imageRec);
	
void  CopyImageRec (void *imageRec,
				 	void **copyRec);

void  ClearImageRec (void **imageRec);
			   				  
void  DisposImageRec (void *imageRec);



	  // File I/O support
	  
void  PackSymbol (short symCode, Handle symBuf, unsigned short *symBytes);

void  UnpackSymbol (short symCode, Handle symBuf, unsigned short symBytes);

	  
			   