/****
 * CImgWindow.c
 *
 *	Image window methods for the SignWriter Image editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/

#include "CImgWindow.h"
#include "Global.h"
#include "Commands.h"
#include "Controls.h"
#include "CBartender.h"
#include "CCharGrid.h"
#include "CDesktop.h"
#include "CImgWndPane.h"
#include "CPaneBorder.h"
#include "CWindow.h"
#include "SymEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"
#include <constants.h>
#include <BDC.h>



extern	CBartender	*gBartender;	/* The menu handling object */
extern	CDesktop	*gDesktop;		/* The enclosure for all windows */
extern	CBureaucrat	*gGopher;		/* The current boss in the chain of command */




/***
 *  IImgWindow
 *
 *  Initialize a new instance of a Image window.
 *
 ***/
 
void CImgWindow::IImgWindow (CDirectorOwner *aSupervisor)

{
	inherited::IDirector(aSupervisor);
}



/***
 * Dispose
 *
 *
 ***/

void CImgWindow::Dispose()

{
	inherited::Dispose();
}



/***
 * DoCommand
 *
 *
 ***/

void CImgWindow::DoCommand (long theCommand)

{
	switch (theCommand) {
						
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * UpdateMenus
 *
 *
***/

 void CImgWindow::UpdateMenus()

{
  	inherited::UpdateMenus();
}



/***
 * NewWindow
 *
 *	Build a new instance of an image window and display
 *	the data specified by the argument.
 *
 ***/

void CImgWindow::NewWindow (short symCode, short symVar, short fillVar, short image,
							CSymWndGrid *symWind)
{
	CCharGrid		*theToolPane;
	CPaneBorder		*theEditBorder;
	CPaneBorder		*theToolBorder;

	Rect			wLoc;
	static short	wCount = 0;
	
	Str27			winTitle = "\pS";
	Str15			varPart  = "\p, V";
	Str15			fillPart = "\p, F";
	Str15			imgPart  = "\p, I";
	Str15			numStr;
	
	
	   /**
		**	Get the (staggered) window location.
		**
		**/
		
	SetRect(&wLoc, IWND_LEFT, IWND_TOP, 
				   IWND_LEFT + IWND_WIDTH, IWND_TOP + IWND_HEIGHT);
				   
	OffsetRect(&wLoc, wCount * HORIZ_WOFFSET, wCount * VERT_WOFFSET);
	wCount = (wCount + 1) % MAX_WOFFSETS;
	
	
	   /**
		**	Make the (invisible) window.
		**
		**/
		
	itsWindow = new CWindow;
	itsWindow->INewWindow(&wLoc, FALSE, 4, FALSE, TRUE, gDesktop, this);
	
	
		/**
		 **  Set the window title to "Sw, Vx, Fy, Iz"
		 **
		 **/
		 
	NumToString(symCode, numStr);
	ConcatPStrings(winTitle, numStr);

	ConcatPStrings(winTitle, varPart);

	NumToString(symVar, numStr);
	ConcatPStrings(winTitle, numStr);

	ConcatPStrings(winTitle, fillPart);

	NumToString(fillVar, numStr);
	ConcatPStrings(winTitle, numStr);

	ConcatPStrings(winTitle, imgPart);

	NumToString(image, numStr);
	ConcatPStrings(winTitle, numStr);

	itsWindow->SetTitle(winTitle);

	
		/**
		 **	itsEditPane controls the bitmap at the center
		 ** of the Image window.  It is used to edit the 
		 ** bitmap that constitutes the selected image.
		 **
		 **/

	itsEditPane = new CImgWndPane;
    itsEditPane->IImgWndPane(itsWindow, this,
							 IPANE_WIDTH, IPANE_HEIGHT,
							 IPANE_LEFT, IPANE_TOP,
							 sizFIXEDLEFT, sizFIXEDBOTTOM,
    						 symCode, symVar, fillVar, image,
    						 symWind);
    						 
		
		/**
		 **	theEditBorder is used to draw the bottom
		 ** border line on the edit pane.
		 **
		 **/
		 
	theEditBorder = new CPaneBorder;
	theEditBorder->IPaneBorder(kBorderTop);
	
	itsEditPane->SetBorder(theEditBorder);
	

		/**
		 **	theToolPane controls the tools that appear
		 ** across the bottom of the Image window. It is 
		 ** used to change the grid window's draw mode.
		 **
		 **/
		 
	theToolPane = new CCharGrid;
    theToolPane->ICharGrid(ChGdTools, itsWindow, itsEditPane);
    theToolPane->selection = toolFRAME;
    
    
    	/**
		 **	theToolBorder is used to draw the lefthand
		 ** border line on the tool pane.
		 **
		 **/
		 
	theToolBorder = new CPaneBorder;
	theToolBorder->IPaneBorder(kBorderRight);

	theToolPane->SetBorder(theToolBorder);
	

		/**
		 **
		 ** The edit pane fields all image window
		 ** commands.
		 ** 
		 **/
		 
	itsEditPane->SetToolPane(theToolPane);
	itsEditPane->BecomeGopher(TRUE);
	itsGopher = itsEditPane;
		
	
		/**
		 **	Link the new image window to its symbol window.
		 **
		 **/
		 
	 itsSymWind = symWind;
	 itsSymWind->SetImageWindow(symVar, fillVar, image, this);
								
								
		/**
		 **	Send the window a Select() message to make
		 **	it the active window.
		 **
		 **/
	
	itsWindow->Select();
}



/***
 * Close (override)
 *
 *	Save current image before closing image window
 *
 ***/
 
Boolean	CImgWindow::Close (Boolean quitting)
{
	// save image
	if (!itsEditPane->Close())
		return FALSE;
	
	// unlink from symbol window
	itsSymWind->SetImageWindow(itsEditPane->itsSymVar, itsEditPane->itsFillVar, 
							   itsEditPane->itsImage, NULL);
	
	return inherited::Close(quitting);
}
	
