/****
 * CSymWndBox.c
 *
 *	Symbol window box methods for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 *
 ****/


#include "CSymWndBox.h"
#include "CSymWndGrid.h"
#include "SymEditConsts.h"
#include "Symbols.h"
#include <CPane.h>
#include <BDC.h>
#include "TBUtilities.h"




void CSymWndBox::ISymWndBox(CView *anEnclosure, CBureaucrat *aSupervisor,
								short aWidth, short aHeight,
								short aHEncl, short aVEncl,
								SizingOption aHSizing, SizingOption aVSizing,
								short symCode)
{
	itsSymCode = symCode;
	
	inherited::IPane(anEnclosure, aSupervisor, aWidth, aHeight,
					aHEncl, aVEncl, aHSizing, aVSizing);
}
							

/***
 * Draw
 *
 *	Draw the type name of the symbol in the
 *  enclosing symbol window.
 *
 ***/

void CSymWndBox::Draw(Rect *area)

{
	DrawSymType();
}



/***
 * Update
 *
 *	Update the type name of the symbol in the enclosing
 *  symbol window.   Used only when changing the type
 *  of an existing symbol.
 *
 ***/

void CSymWndBox::Update(void)

{
	Prepare();
	DrawSymType();
}



/***
 * DrawSymType
 *
 *	Draw the typename of the symbol
 *	in the enclosing symbol window.  
 *
 ***/
  
static Str15 name[] = { 

"\pFace", "\pHand", "\pMovement", "\pShoulder", "\pContact", "\pOther"	

};

void CSymWndBox::DrawSymType(void)

{
	Rect		 rect;
	SymbolType   type;

	SetRect(&rect, 0, 0, SBOX_WIDTH, SBOT_HEIGHT); 
	EraseRect(&rect);
	
	GetSymType(itsSymCode, &type);
	
	TextFont(1); TextFace(0); TextSize(10);
	MoveTo((SBOX_WIDTH - StringWidth(name[type])) / 2, SBOT_HEIGHT - 4);
	DrawString(name[type]); 
}



/***
 * Dispose
 *
 *	Pass the buck.
 *
 ***/

void CSymWndBox::Dispose(void)

{
	inherited::Dispose();
}


