/******************************************************************************
 CToolShapes.h

		Interface for the ToolShapes Class
		
		Also contains classes for the individual shape tools
		which implement only one method -- UseTool.
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
 ******************************************************************************/
 
#define _H_CToolShapes

#include "CPaintTask.h"					/* Interface for its superclass		*/
#include "CPaintPane.h"

struct CToolShapes : CPaintTask {		/* Class Declaration				*/
								/** Instance Variables **/

								/** Instance Methods **/
									/** Contruction/Destruction **/
	void		IToolShapes(CPaintPane *aPaintPane, CBitMap *aPainting);

	void		BeginTracking(LongPt *startPt);
	void		KeepTracking(LongPt *currPt, LongPt *prevPt, LongPt *startPt);
	void		EndTracking(LongPt *currPt, LongPt *prevPt, LongPt *startPt);
	void		TweakPoints(LongPt *currPt, LongPt *prevPt, LongPt *startPt);
	void		CalcChangedRect(LongPt *currPt, LongPt *prevPt, LongPt *startPt,
									LongRect *changedRect);
	void		UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};


					/*** Classes for the Individual Tools ***/

struct CToolRect : CToolShapes {		/* Rectangle						*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};

struct CToolFillRect : CToolShapes {	/* Filled Rectangle					*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};

struct CToolRRect : CToolShapes {		/* Rounded Rectangle				*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};

struct CToolFillRRect : CToolShapes {	/* Filled Rounded Rectangle			*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};	

struct CToolOval : CToolShapes {		/* Oval								*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};

struct CToolFillOval : CToolShapes {	/* Filled Oval						*/
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
};

struct CToolLine : CToolShapes {
	Point	lineStart;
	Point	lineEnd;
	
	void	UseTool(LongRect *currRect, LongPt *currPt, LongPt *startPt);
	void	TweakPoints(LongPt *currPt, LongPt *prevPt, LongPt *startPt);
	void	CalcChangedRect(LongPt *currPt, LongPt *prevPt, LongPt *startPt,
							LongRect *changedRect);
};
