/******************************************************************************
 CArtClassDoc.c

							The ArtClassDoc Class
	
	SUPERCLASS = CDocument
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <Constants.h>
#include <Commands.h>
#include <CScrollPane.h>
#include <CDesktop.h>
#include <CBartender.h>
#include <CDecorator.h>
#include <CPrinter.h>
#include <CPNTGFile.h>
#include <CApplication.h>
#include <TBUtilities.h>
#include "CArtClassDoc.h"
#include "ArtClassCmds.h"
#include "CPaintPane.h"
#include "CWindow.h"
#include <Packages.h>

/*** Global Variables ***/

extern CApplication	*gApplication;		/* Application object				*/
extern CDesktop		*gDesktop;			/* The visible Desktop				*/
extern CBartender	*gBartender;		/* Manages all menus				*/
extern CDecorator	*gDecorator;		/* Decorator for arranging windows	*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern OSType		gSignature;			/* Creator for Application's files	*/
extern short		gPaintTool;
extern short		gLineSize;

extern short		gFontItem;			/* Font menu item number			*/
extern short		gSizeItem;			/* Size menu item number			*/
extern Style		gFontStyle;			/* Current font style				*/
extern long			gLineSpacing;		/* Line spacing cmd number			*/

/*** Class Constants ***/

#define		WINDmyDoc		1000		/* Resource ID for document	windows	*/
#define		DLOG_PRINT		2000		/* Dialog to display when printing	*/
#define		strABORTPRINT	8			/* String index in STRcommon		*/


/******************************************************************************
 IArtClassDoc

		Initialize an ArtClassDoc object
 ******************************************************************************/

void	CArtClassDoc::IArtClassDoc(
    CApplication        *aSupervisor)
{
	CDocument::IDocument(aSupervisor, TRUE);
}
	

/******************************************************************************
 DoCommand {OVERRIDE}

		Execute a command
 ******************************************************************************/

void	CArtClassDoc::DoCommand(
	long		theCommand)
{
	switch (theCommand) {
	
		case cmdOne:					/* Debugging commands				*/
			break;						/* Add code here to experiment		*/
			
		case cmdTwo:
			break;
			
		case cmdThree:
			break;
			
		default:
			inherited::DoCommand(theCommand);
			break;
	}
}
	

/******************************************************************************
 UpdateMenus {OVERRIDE}

		Perform menu management tasks
 ******************************************************************************/

void	CArtClassDoc::UpdateMenus()
{
	MenuHandle		macMenu;

	inherited:: UpdateMenus();
	
	if (gGopher != itsMainPane) {
		return;
	}
	
	if (gFontItem > 0) {
		macMenu = GetMHandle(MENUfont);
		if (macMenu != NULL) {
			CheckItem(macMenu, gFontItem, TRUE);
		}
	}
	
	if (gSizeItem > 0) {
		macMenu = GetMHandle(MENUsize);
		if (macMenu != NULL) {
			CheckItem(macMenu, gSizeItem, TRUE);
		}
	}
	
	if (gFontStyle == NOTHING) {
		gBartender->CheckMarkCmd(cmdPlain, TRUE);
	} else {
		if (gFontStyle & bold) {
			gBartender->CheckMarkCmd(cmdBold, TRUE);
		}
		if (gFontStyle & italic) {
			gBartender->CheckMarkCmd(cmdItalic, TRUE);
		}
		if (gFontStyle & underline) {
			gBartender->CheckMarkCmd(cmdUnderline, TRUE);
		}
		if (gFontStyle & outline) {
			gBartender->CheckMarkCmd(cmdOutline, TRUE);
		}
		if (gFontStyle & shadow) {
			gBartender->CheckMarkCmd(cmdShadow, TRUE);
		}
		if (gFontStyle & condense) {
			gBartender->CheckMarkCmd(cmdCondense, TRUE);
		}
		if (gFontStyle & extend) {
			gBartender->CheckMarkCmd(cmdExtend, TRUE);
		}
	}

	gBartender->CheckMarkCmd(gLineSpacing, TRUE);
}
	

/******************************************************************************
 NewFile {OVERRIDE}

		Set up a document with a new file, usually in response the "New"
		command in the File menu.
 ******************************************************************************/

void	CArtClassDoc::NewFile()
{
	Str255		wTitle;
	Str255		wNumber;
	short		wCount;
	
										/* Build a new window				*/
	BuildWindow(NULL);
	

	itsWindow->GetTitle(wTitle);	/* Append an index number to the	*/
									/*   default name of the window		*/
	wCount = gDecorator->GetWCount();
	NumToString((long)wCount, wNumber);
	ConcatPStrings(wTitle, (StringPtr) "\p-");
	ConcatPStrings(wTitle, wNumber);
	itsWindow->SetTitle(wTitle);
	
	itsWindow->Select();			/* Make it the active window		*/
		
}
	

/******************************************************************************
 OpenFile {OVERRIDE}

		Open a file for a document, usually in response the "Open"
		command in the File menu.
 ******************************************************************************/

void	CArtClassDoc::OpenFile(
	SFReply		*macSFReply)
{
	CPNTGFile	*theFile = NULL;
	CBitMap		*theBitMap = NULL;
	OSErr		macErr;

	TRY
	{
		theFile = new(CPNTGFile);			/* Make a File object and read the	*/
		itsFile = theFile;
		theFile->IPNTGFile();				/*   data into a new BitMap			*/
		theFile->SFSpecify(macSFReply);
		
		
		theFile->Open(fsRdWrPerm);
										/* File was opened OK, now try to	*/
										/*   read bit map into memory. It's	*/
										/*   alright for this memory		*/
										/*   request to fail.				*/
		theBitMap = theFile->ReadNewBitMap(TRUE);
		
		FailNIL( theBitMap);
										/* Make a window which displays		*/
										/*   the BitMap we just read in		*/
		
		BuildWindow(theBitMap);
										/* Set window title to the name of	*/
										/*   the file						*/
		itsWindow->SetTitle(macSFReply->fName);
	
		itsWindow->Select();		/* Make this the active window		*/

	}
	CATCH
	{ 									/* Not enough memory to open Doc	*/
		if (gLastError == memFullErr)
			SetFailInfo( memFullErr, SpecifyMsg( 1024, 1));
	}
	ENDTRY;
}
	

/******************************************************************************
 BuildWindow

		Create the window and panes for a painting document
 ******************************************************************************/

void		CArtClassDoc::BuildWindow(
	CBitMap		*theBitMap)
{
	CScrollPane	*theScrollPane;
	CPaintPane	*thePaintPane = NULL;
	LongRect	bounds;
	Rect		sizeRect;
	
	TRY
	{
		itsWindow = new(CWindow);			/* Create window object				*/
		itsWindow->IWindow(WINDmyDoc, FALSE, gDesktop, this);
		SetRect(&sizeRect, 100, 100, PNTG_WIDTH + SBARSIZE1, PNTG_HEIGHT + SBARSIZE1);
		itsWindow->SetSizeRect(&sizeRect);
		gDecorator->PlaceNewWindow(itsWindow);
											/* Create a ScrollPane in the window */
		theScrollPane = new(CScrollPane);
		theScrollPane->IViewRes('ScPn', 1, itsWindow, this);
		theScrollPane->FitToEnclFrame(TRUE, TRUE);
		
											/* Put our PaintPane inside the		*/
											/*   ScrollPane						*/
		SetLongRect(&bounds, 0, 0, PNTG_WIDTH, PNTG_HEIGHT);
		thePaintPane = new(CPaintPane);
		thePaintPane->IPaintPane(theScrollPane, this, 0, 0, 0, 0,
									sizELASTIC, sizELASTIC, &bounds, theBitMap);
			
										/* Set up panes within window		*/
		thePaintPane->FitToEnclosure(TRUE, TRUE);
		theScrollPane->InstallPanorama(thePaintPane);
		itsMainPane = thePaintPane;
		itsGopher = thePaintPane;
		
		theScrollPane->SetSteps(8, 8);	/* Set scrolling parameters			*/
		theScrollPane->SetOverlaps(72, 72);
	}
	CATCH
	{
			/* if we couldn't create the paint pane, but there's a	*/
			/* bit map, then dispose of it.							*/
		if (!thePaintPane && theBitMap)
			theBitMap->Dispose();
	}
	ENDTRY;
}


/**** F I L I N G   M E T H O D S ****/
	

/******************************************************************************
 DoSave {OVERRIDE}

		Save a file under the same name.
 ******************************************************************************/

Boolean	CArtClassDoc::DoSave()
{
	CBitMap		*theBitMap;
	
	if (itsFile == NULL) {				/* Same as SaveAs if there's no		*/
		return(DoSaveFileAs());			/*   existing file					*/
	
	} else {
										/* Get pane ready to be saved		*/
		((CPaintPane*)itsMainPane)->KillSelRect();
										/* Write out bit map to disk as a	*/
										/*   MacPaint format file			*/
		theBitMap = ((CPaintPane*)itsMainPane)->GetBitMap();
		((CPNTGFile*)itsFile)->WriteBitMap(theBitMap);
		dirty = FALSE;
		return(TRUE);
	}
}
	

/******************************************************************************
 DoSaveAs {OVERRIDE}

		Save a file under another name.
 ******************************************************************************/

Boolean		CArtClassDoc::DoSaveAs(
	SFReply		*macSFReply)
{
	if (itsFile != NULL) {				/* Throw out existing file object	*/
		itsFile->Dispose();
	}
	
	itsFile = new(CPNTGFile);			/* Make a new file object with the	*/
	((CPNTGFile*)itsFile)->IPNTGFile();	/*   name from the SFReply record	*/
	itsFile->SFSpecify(macSFReply);
	
	if (itsFile->ExistsOnDisk())		/* delete old file if it exists		*/
		itsFile->ThrowOut();
		
	itsFile->CreateNew(gSignature, 'PNTG');
	itsFile->Open(fsRdWrPerm);

	itsWindow->SetTitle(macSFReply->fName);

	return( DoSave() );					/* Perform the actual write to disk	*/
}
	

/******************************************************************************
 DoRevert {OVERRIDE}

		Throw out the current contents of a document and revert to the
		last saved version.
 ******************************************************************************/

void	CArtClassDoc::DoRevert()
{
	CBitMap		*theBitMap;
	LongPt		homePos;
	
	itsFile->Close();					/* Close file and get rid of the	*/
										/*   offscreen bit image			*/
	theBitMap = ((CPaintPane*)itsMainPane)->GetBitMap();
	theBitMap->Dispose();
	
	itsFile->Open(fsRdWrPerm);			/* Reopen file and read original	*/
										/*   bit map from disk				*/
	theBitMap = ((CPNTGFile*)itsFile)->ReadNewBitMap(TRUE);
	((CPaintPane*)itsMainPane)->SetBitMap(theBitMap);
	
										/* Reset window to home position	*/
	((CPaintPane*)itsMainPane)->GetHomePosition(&homePos);
	((CPaintPane*)itsMainPane)->ScrollTo( &homePos, FALSE);
	
										/* Get rid of any active selection	*/
	SetLongRect(&((CPaintPane*)itsMainPane)->selRect, 0, 0, 0, 0);
	itsMainPane->Refresh();
	
	dirty = FALSE;
}
	

/******************************************************************************
 AboutToPrint

		The specified range of pages is about to be printed
 ******************************************************************************/

void	CArtClassDoc::AboutToPrint(
	short		*firstPage,
	short		*lastPage)
{
	Str255		docName;
	
										/* Get pane ready to be printed		*/
	((CPaintPane*)itsMainPane)->KillSelRect();
	
	inherited::AboutToPrint(firstPage, lastPage);
	
										/* Display banner to let user know	*/
										/*   printing is taking place		*/
	PositionDialog('DLOG', DLOG_PRINT);
	printBanner = GetNewDialog(DLOG_PRINT, NULL, (WindowPtr) -1L);
	DrawDialog(printBanner);
	
	GetName(docName);					/* Document name shown in status	*/
	SetWTitle(printBanner, docName);	/*   box when printing to a			*/
										/*   LaserWriter is taken from the	*/
										/*   title of the top window, which	*/
										/*   is now our dialog box.			*/
}
	

/******************************************************************************
 DonePrinting {OVERRIDE}

		Print loop has been completed
 ******************************************************************************/

void	CArtClassDoc::DonePrinting()
{
	short		itemType;
	Handle		item;
	Rect		box;
	Str63		abortStr;
	long		ticks;
	
	if (PrError() == iPrAbort) {
		GetDItem(printBanner, 1, &itemType, &item, &box);
		GetIndString(abortStr, STRcommon, strABORTPRINT);
		SetIText(item, abortStr);
		Delay(120, &ticks);
	}
		
	DisposDialog(printBanner);

	inherited::DonePrinting();
}
