/******************************************************************************
 CToolText.c

							The ToolText Class
							
		A typical text tool for typing text in a Painting. A Caption, which
		is a subclass of CEditText, is created to handle the text. This
		actually becomes a subpane of the painting. The Pane itself is
		disposed after the typing is completed, but the bits within the
		Pane become part of the painting.
		
	SUPERCLASS = CPaintTask
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <CDirector.h>
#include <CPNTGFile.h>
#include "CToolText.h"

/*** Global Variables ***/

extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern long			gSleepTime;			/* Max time between events			*/
extern CBitMap		*gScratchPad;

extern short		gFontNum;			/* Font number						*/
extern short		gFontSize;			/* Font size						*/
extern Style		gFontStyle;			/* Current font style				*/
extern long			gLineSpacing;		/* Line spacing cmd number			*/

/*** Class Constants ***/

#define		iTEXT			11			/* Index for Text Task name			*/


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IToolText

		Initialize a ToolText object
 ******************************************************************************/

void	CToolText::IToolText(
	CPaintPane	*aPaintPane,
	CBitMap		*aPainting)
{
	CPaintTask::IPaintTask(iTEXT, aPaintPane, aPainting);
	
	itsCaption = NULL;
	done = FALSE;
}

/******************************************************************************
 BeginTracking {OVERRIDE}

		Initial mouse down determines the location for the text
 ******************************************************************************/

void	CToolText::BeginTracking(
	LongPt		*startPt)
{
										/* Create a Caption Pane to handle	*/
										/*   keystrokes and text display	*/
										
	TextFont(gFontNum);					/* Set font characteristics			*/
	TextSize(gFontSize);
	TextFace(gFontStyle);									
										/* Note how the PaintPane is both	*/
										/*   the enclosure and supervisor	*/
										/*   of the Caption. This is		*/
										/*   important because we want the	*/
										/*   PaintPane to become the Gopher	*/
										/*   again when we get rid of the	*/
										/*   Caption pane.					*/
	itsCaption = new(CCaption);
	itsCaption->ICaption(itsPaintPane, itsPaintPane,
							PNTG_WIDTH, startPt->h, startPt->v);
							
	itsCaption->SetSpacingCmd(gLineSpacing);

	itsCaption->Activate();
	itsCaption->BecomeGopher( TRUE);
}


/******************************************************************************
 Do {OVERRIDE}

		Finish off Text
 ******************************************************************************/

void	CToolText::Do()
{
	LongRect	tFrame;
	
	if (!done) {						/* We have to save stuff for Undo,	*/
										/*   copy the image from the		*/
										/*   Caption to the Painting, then	*/
										/*   throw out the Caption.			*/
		itsCaption->Deactivate();
		itsCaption->GetFrame(&tFrame);
		itsCaption->FrameToEnclR(&tFrame);
		
		SaveForUndo(&tFrame);
		
		itsCaption->DrawInPort(itsPainting->macPort);
		itsCaption->Dispose();
		itsCaption = NULL;
		itsPaintPane->BecomeGopher( TRUE);
		done = TRUE;
	}
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo typing of text
 ******************************************************************************/

void	CToolText::Undo()
{
	if (!done) {						/* We must be sure to finalize the	*/
		Do();							/*   typing before undoing it.		*/
	}
	
	inherited::Undo();
}
