/******************************************************************************
 CDemoTableDoc.h

		
	SUPERCLASS = CDocument
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CDemoTableDoc.h"
#include "CDemoTable.h"
#include "CTableScroller.h"
#include "CDecorator.h"
#include "Constants.h"
#include "CDLOGDialog.h"
#include "CClickCmdPane.h"
#include "CPaneBorder.h"
#include "CTableLabels.h"
#include "Commands.h"

#define	kTableWIND			1024		// Resource ID for WIND template

#define	kInitialRows		1000		// number of rows in the table
#define kInitialCols		10			// number of columns in the table


extern struct CApplication 	*gApplication;
extern struct CDesktop		*gDesktop;
extern CDecorator			*gDecorator;

/******************************************************************************
 IDemoTableDoc
 
 	Initialize the document.
******************************************************************************/

void CDemoTableDoc::IDemoTableDoc( void)
{
	itsTable = NULL;

	CDocument::IDocument( gApplication, TRUE);
}

/******************************************************************************
 DoCommand
******************************************************************************/

void CDemoTableDoc::DoCommand( long theCommand)
{
	switch( theCommand)
	{
		default:
			inherited::DoCommand( theCommand);
			break;
	}
}

/******************************************************************************
 BuildWindow
 
 	Create the window for this document.
******************************************************************************/

void CDemoTableDoc::BuildWindow(void)
{
	Rect			sizeRect;
	CTableScroller 	*theScrollPane;
	CClickCmdPane	*selectAllPane;
	CPaneBorder		*paneBorder;
	CTableLabels	*rowLabel, *colLabel;
	
		// create a new window from a WIND resource
	
	itsWindow = new(CWindow);
	itsWindow->IWindow( kTableWIND, FALSE, gDesktop, this);
	
		// create a CScrollPane subclass for scrolling row and column labels

	theScrollPane = new(CTableScroller);
	theScrollPane->ITableScroller(itsWindow, this, 0, 0, 0, 0,
						sizELASTIC, sizELASTIC, TRUE, TRUE, TRUE);
	theScrollPane->FitToEnclFrame(TRUE, TRUE);
	
		// The order of events here is a little unusual. Panes are
		// inserted into their enclosure's subview list when CPane::IPane
		// is called. The order in the subview list determines the order
		// in which panes are drawn. We want the row and column labels 
		// and the selectAll pane inserted before the table because the
		// resulting visual appearance is better. But we need to set the
		// supervisor of the selectAll pane to the table. To accomplish
		// this we create the table object first, but delay calling
		// its initialization method until the other three panes are initialized.
		
	itsTable = new CDemoTable;
	
		// the selectAllPane is a simple CPane subclass that sends
		// its supervisor a DoCommand message when clicked. This
		// is used to send the table a cmdSelectAll.
	
	selectAllPane = new( CClickCmdPane);
	selectAllPane->IClickCmdPane( theScrollPane, itsTable, kRowLabelWidth,
			kColLabelHeight, 0, 0, sizFIXEDLEFT, sizFIXEDTOP, cmdSelectAll);
			
		// create a rectangular border for the selectAll pane
			
	paneBorder = new( CPaneBorder);
	paneBorder->IPaneBorder( kBorderFrame);
	selectAllPane->SetBorder( paneBorder);
	
		// create the row and column labels

	rowLabel = new CTableLabels;
	rowLabel->ITableLabels( theScrollPane, v, kRowLabelWidth, kColLabelHeight);
	colLabel = new CTableLabels;
	colLabel->ITableLabels( theScrollPane, h, kColLabelHeight, kRowLabelWidth);
	
		// finally initialize the table

	itsTable->IDemoTable( theScrollPane, this, rowLabel, colLabel);
	
		// insert some rows and columns into the new table
	
	itsTable->AddRow( kInitialRows, -1);
	itsTable->AddCol( kInitialCols, -1);
	
		// link the labels with the scroll pane
	
	theScrollPane->SetLabels( rowLabel, colLabel);
	
		// itsGopher is the object that becomes first in line for
		// commands, keystrokes, and idle time when the document is
		// active.
		
		// itsMainPane is the pane that prints if a document is printable

	itsGopher = itsMainPane = itsTable;
	
		// link the table to the scroll pane
	
	theScrollPane->InstallPanorama( itsTable);
	
		// place and size the window for the main screen
		
	gDecorator->PlaceNewWindow( itsWindow);
	
		// show the window
		
	itsWindow->Select();
}

/******************************************************************************
 NewFile
 
 	Called when the user has created a new document. There's no data behind
 	this document, so we just create a window.
 	
******************************************************************************/

void CDemoTableDoc::NewFile(void)
{
	BuildWindow();
}

/******************************************************************************
 OpenFile
 
 	Doesn't open any files.
******************************************************************************/

void CDemoTableDoc::OpenFile(SFReply *macSFReply)
{

}

/******************************************************************************
 DoSave
 
 	Doesn't save
******************************************************************************/

Boolean	CDemoTableDoc::DoSave(void)
{

}

/******************************************************************************
 DoSaveAs
 
 	Doesn't Save as
******************************************************************************/

Boolean	CDemoTableDoc::DoSaveAs(SFReply *macSFReply)
{

}

/******************************************************************************
 DoRevert
 
 	Doesn't revert
******************************************************************************/

void CDemoTableDoc::DoRevert(void)
{

}

/******************************************************************************
 UpdateMenus
 
 	No special menu updating to perform.
******************************************************************************/

void CDemoTableDoc::UpdateMenus(void)
{
	inherited::UpdateMenus();
}
