/*****
 * TBullWindow.c
 *
 *	Copyright  1991 Symantec Corporation. All rights reserved.
 *
 *		The window methods for the Object Bullseye demo.
 *
 *		TBullWindow inherits its Draw method from TWindow.
 *		TSquareBull and TCircleBull override DrawShape to draw their own kind of bullseye.
 *
 *****/

#include <Packages.h>			// for NumToString prototype
#include "TBullWindow.h"


/***
 * concatenate two Pascal strings
 *
 ***/

static concat (StringPtr s, StringPtr p)
{
	BlockMove(p+1, s+1+s[0], (long) p[0]);
	s[0] += p[0];	
}


/****
 *	TBullWindow constructor
 *
 *		Create a bullseye window. This constructor relies
 *		on the TWindow constructor to place the window in
 *		in attractive place. Then it appends a number to the
 *		window title.
 *
 ****/

TBullWindow::TBullWindow(void)
{
	Str15	numStr;
	Str255	title;
	long	windowNumber;
	
	
		//	Set the title of the window to be
		//	the title in the resource
		//	plus the window counter.
		//	The window counter is a class variable
		//	declared in the TWindow class.


	GetWindowTitle(title);
	windowNumber = GetWindowNumber();
	NumToString(GetWindowNumber(), numStr);
	concat(title, numStr); 
	SetWindowTitle(title);

		//	Set the width, and show the window.
		
	SetWidth(5);
	Show();
}



/****
 * Hit
 *
 *		Handle a mouse down in the window.
 *		Bullseye window just force a refresh.
 *
 ****/

short TBullWindow::Hit(Point where)
{
	RefreshWindow(false);		// preserve the scroll bars
}


/****
 * GetWidth
 * SetWidth
 *
 *		Get and set the width of the bullseye figures.
 *
 ****/

short	TBullWindow::GetWidth()
{
	return width;
}


short	TBullWindow::SetWidth(short w)
{
	width = w;
	RefreshWindow(true);
}


/****
 * Draw
 *
 *		Draw the bullseye figures.
 *		Repeatedly call DrawShape with a smaller drawing area.
 *		The drawing area gets smaller by 2 * the width.
 *
 ****/

short	TBullWindow::Draw(void)

{
	PenState	pen;
	RgnHandle	saveClip = NewRgn();
	Rect		drawingRect;
	
	GetPenState(&pen);
	GetClip(saveClip);
	
	inherited::Draw();
	
	GetWindowRect(&drawingRect, false);		// Don't draw in the scroll
											// bar areas. Note that it's
											// ok to pass the address of
											// drawingRect because
											// GetWindowRect won't move
											// memory.
	
	PenSize(width, width);
	ClipRect(&drawingRect);
	
	while (drawingRect.left < drawingRect.right &&
		   drawingRect.top  < drawingRect.bottom) {
		DrawShape(&drawingRect);
		InsetRect(&drawingRect, width*2, width*2);
	}

	SetPenState(&pen);
	SetClip(saveClip);
	DisposeRgn(saveClip);
}


/****
 * DrawShape methods
 *
 *		These are the DrawShape methods for
 *			TBullWindow:	does nothing
 *			TCircleBull:	Circles
 *			TSquareBull:	Squares
 *			TTriangleBull:	"Triangles"
 *
 *		All the DrawShape methods take a drawingRect
 *		as a parameter. The pen width
 *		is already set to the appropriate width.
 *
 ****/

short TBullWindow::DrawShape(Rect *drawingRect)
{
}


short TCircleBull::DrawShape(Rect *drawingRect)
{
	FrameOval(drawingRect);
}


short TSquareBull::DrawShape(Rect *drawingRect)
{
	FrameRect(drawingRect);
}




short TTriangleBull::DrawShape(Rect *drawingRect)
{
	MoveTo(drawingRect->left, drawingRect->bottom);	
	LineTo(drawingRect->right, drawingRect->bottom);
	LineTo(drawingRect->left, drawingRect->top);
	LineTo(drawingRect->left, drawingRect->bottom);
}