/*****
 * objBullMenus.c
 *
 *	Copyright  1991 Symantec Corporation. All rights reserved.
 *
 *	Menu handling routines for the Object Bullseye demo program.
 *
 *****/

#include <oops.h>
#include "objBullMenus.h"
#include "TBullWindow.h"


	// prototypes for utility routines
static enable (MenuHandle menu, short item, Boolean enableMenu);
static findColorItem (long color);

MenuHandle	appleMenu,
			fileMenu,
			editMenu,
			widthMenu,
			foreMenu,
			backMenu;

enum	{					// Menu ids
			appleID = 1,
			fileID,
			editID,
			widthID,
			foreID,
			backID
};

enum	{					// items in the File menu
			newItem = 1,
			squareItem,
			triangleItem,
			closeItem,
			quitItem = closeItem+2	// leave space for the dashed line before Quit
};


long colorTable[] = {		// table of the Classic QuickDraw in menu order
			0,
			blackColor,
			whiteColor,
			redColor,
			greenColor,
			blueColor,
			cyanColor,
			magentaColor,
			yellowColor
};




/****
 * SetUpMenus()
 *
 *	Set up the menus.
 *
 ****/

void SetUpMenus(void)

{
	InsertMenu(appleMenu = GetMenu(appleID), 0);
	AddResMenu(appleMenu, 'DRVR');

	InsertMenu(fileMenu = GetMenu(fileID), 0);
	InsertMenu(editMenu = GetMenu(editID), 0);
	InsertMenu(widthMenu = GetMenu(widthID), 0);
	InsertMenu(foreMenu = GetMenu(foreID), 0);
	InsertMenu(backMenu = GetMenu(backID), 0);
	
	DrawMenuBar();
}
/* end SetUpMenus */


/****
 *  AdjustMenus()
 *
 *	Enable or disable the items in the Edit menu if a DA window
 *	comes up or goes away. Our application doesn't do anything with
 *	the Edit menu. If one of our windows is frontmost, adjust the
 *	Width and color menus to reflect the proper values for the
 *	windows.
 *
 ****/

#define	isEnabled(mH)	((**mH).enableFlags & 0x01)

void AdjustMenus(void)
{
	register WindowPeek wp = (WindowPeek) FrontWindow();
	short kind = wp ? wp->windowKind : 0;
	short	i;
	short	max;
	static short	redrawForEdit = true;	// did the Edit menu change?
	static short	redrawForWidth = true;	// did the Width menu change?

	if (kind < 0) {							// enable Edit for desk accessories
	  redrawForEdit = !isEnabled(editMenu);
	  enable(editMenu, 0, true);
	}
	else {									// disable it for all others
	  redrawForEdit = isEnabled(editMenu);
	  enable(editMenu, 0, false);
	}

	// enable the Close command for DAs and our windows
	enable(fileMenu, closeItem, kind < 0 || kind == TWINDOWKIND);

	// disable the Width and color menus if our window isn't frontmost
	if (kind != TWINDOWKIND) {
	  redrawForWidth = isEnabled(widthMenu);
	  enable(widthMenu, 0, false);
	  enable(foreMenu, 0, false);
	  enable(backMenu, 0, false);
	}
	else {
	  redrawForWidth = !isEnabled(widthMenu);
	  enable(widthMenu, 0, true);
	  enable(foreMenu, 0, true);
	  enable(backMenu, 0, true);

	  max = CountMItems(widthMenu);			// uncheck all the widths
	  for (i = 1; i <= max; i++)
	    CheckItem(widthMenu, i, false);

	  max = CountMItems(foreMenu);			// uncheck all the colors
	  for (i = 1; i <= max; i++) {
	  	CheckItem(foreMenu, i, false);
	  	CheckItem(backMenu, i, false);
	  }

		// check the width and the colors for the frontmost window
	  CheckItem(widthMenu, OBJ(wp)->GetWidth(), true);
	  CheckItem(foreMenu, findColorItem(OBJ(wp)->GetFore()), true);
	  CheckItem(backMenu, findColorItem(OBJ(wp)->GetBack()), true);
	}

	if (redrawForEdit || redrawForWidth) {
	  DrawMenuBar();
	  redrawForEdit = redrawForWidth = false;
	}
}

/**
 *	enable or disable a menu
 *
 **/

static enable (MenuHandle menu, short item, Boolean enableMenu)
{
	if (enableMenu)
		EnableItem(menu, item);
	else
		DisableItem(menu, item);
}

/**
 *	given an old QuickDraw color, find its menu item number
 *
 **/

static findColorItem (long color)
{
	short	i;
	short max = sizeof(colorTable) / sizeof(long);

	for (i= 1; i <= max; i++)
	  if (colorTable[i] == color)
	    return i;

	DebugStr("\pBAD COLOR!");
}

/*****
 * HandleMenu(mSelect)
 *
 *	Handle the menu selection. mSelect is what MenuSelect() and
 *	MenuKey() return: the high word is the menu ID, the low word
 *	is the menu item
 *
 *****/

void HandleMenu (long mSelect)
{
	short			menuID = HiWord(mSelect);
	short			menuItem = LoWord(mSelect);
	Str255		name;
	GrafPtr		savePort;
	TBullWindow	*bullWindow;
	WindowPeek	wp;
	
	switch (menuID)
	  {
	  case	appleID:
	  	if (menuItem == 1)
	  	  {
	  	  Alert(128, 0L);
	  	  break;
	  	  }
	 
		GetPort(&savePort);
		GetItem(appleMenu, menuItem, name);
		OpenDeskAcc(name);
		SetPort(savePort);
		break;
	
	  case	fileID:
		switch (menuItem)
		  {
		  case	newItem:
		  	bullWindow = new(TCircleBull);
		  	break;

		  case 	squareItem:
		  	bullWindow = new(TSquareBull);
		  	break;
		  	
		  case	triangleItem:
		  	bullWindow = new(TTriangleBull);
		  	break;
		  
		  case	closeItem:
			if ((wp = (WindowPeek) FrontWindow()) == NULL)
			  break;
			  
			if (wp->windowKind < 0)
			  CloseDeskAcc(wp->windowKind);
			else if (wp->windowKind == TWINDOWKIND)
			  OBJ(wp)->Close();
  			break;
  				  	
		  case	quitItem:
			ExitToShell();
			break;
		  }
		break;
  				
	  case	editID:
		if (!SystemEdit(menuItem-1))
		  SysBeep(5);
		break;
		
	  case	widthID:
		if ((wp = (WindowPeek) FrontWindow()) == 0L ||
									wp->windowKind != TWINDOWKIND)
		  break;

		OBJ(wp)->SetWidth(menuItem);
		break;

	  case	foreID:
	  case	backID:
		if ((wp = (WindowPeek) FrontWindow()) == 0L ||
									wp->windowKind != TWINDOWKIND)
		break;

		if (menuID == foreID)
			OBJ(wp)->SetFore(colorTable[menuItem]);
		else
			OBJ(wp)->SetBack(colorTable[menuItem]);
		break;
	  }
}
/* end HandleMenu */

