
/*
 *  math.h
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#pragma once

extern short __D_INF[], __X_INF[];


#if !__option(thinkc)
#define _NOINTRINSICS_
#define _NOSYNONYMS_
#endif

#if __option(double_8)
typedef long double _double;
#define HUGE_VAL			(* (double *) __D_INF)
#else
typedef double _double;
#define HUGE_VAL			(* (double *) __X_INF)
#endif

#ifndef __SANE__
double atan(double);
double cos(double);
double exp(double);
double fabs(double);
double log(double);
double sin(double);
double sqrt(double);
double tan(double);
#endif

double acos(double);
double asin(double);
double atan2(double, double);
double ceil(double);
double cosh(double);
double floor(double);
double fmod(double, double);
double frexp(double, int *);
double ldexp(double, int);
double log10(double);
double modf(double, double *);
double pow(double, double);
double sinh(double);
double tanh(double);


	/*  68881 intrinsics  */

#if __option(mc68881) && __option(thinkc)

_double _acos(_double)	: 0x1C;
_double _asin(_double)	: 0x0C;
_double _atan(_double)	: 0x0A;
_double _cos(_double)	: 0x1D;
_double _cosh(_double)	: 0x19;
_double _exp(_double)	: 0x10;
_double _fabs(_double)	: 0x18;
_double _log(_double)	: 0x14;
_double _log10(_double)	: 0x15;
_double _sin(_double)	: 0x0E;
_double _sinh(_double)	: 0x02;
_double _sqrt(_double)	: 0x04;
_double _tan(_double)	: 0x0F;
_double _tanh(_double)	: 0x09;

#elif !defined(_NOSYNONYMS_)

#define _acos(x)		acos(x)
#define _asin(x)		asin(x)
#define _atan(x)		atan(x)
#define _cos(x)			cos(x)
#define _cosh(x)		cosh(x)
#define _exp(x)			exp(x)
#define _fabs(x)		fabs(x)
#define _log(x)			log(x)
#define _log10(x)		log10(x)
#define _sin(x)			sin(x)
#define _sinh(x)		sinh(x)
#define _sqrt(x)		sqrt(x)
#define _tan(x)			tan(x)
#define _tanh(x)		tanh(x)

#endif

#ifndef _NOSYNONYMS_
#define _atan2(x, y)	atan2(x, y)
#define _ceil(x)		ceil(x)
#define _floor(x)		floor(x)
#define _fmod(x, y)		fmod(x, y)
#define _frexp(x, y)	frexp(x, y)
#define _ldexp(x, y)	ldexp(x, y)
#define _modf(x, y)		modf(x, y)
#define _pow(x, y)		pow(x, y)
#endif


	/*  map math functions to intrinsics  */

#if !defined(_NOINTRINSICS_) && !defined(__SANE__) && __option(mc68881) && !__option(double_8)

#define atan(x)			_atan(x)
#define cos(x)			_cos(x)
#define fabs(x)			_fabs(x)
#define sin(x)			_sin(x)
#define tanh(x)			_tanh(x)

#ifdef _NOERRORCHECK_
#define acos(x)			_acos(x)
#define asin(x)			_asin(x)
#define cosh(x)			_cosh(x)
#define exp(x)			_exp(x)
#define log(x)			_log(x)
#define log10(x)		_log10(x)
#define sinh(x)			_sinh(x)
#define sqrt(x)			_sqrt(x)
#define tan(x)			_tan(x)
#endif

#endif
