
/*
 *  fseek.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#include "stdio.h"
#include "errno.h"


int
fseek(FILE *fp, long offset, int whence)
{
	long pos = fp->pos - fp->cnt;
	
	if (fp->refnum <= 0) {
		errno = ENODEV;
		return(-1);
	}
	fflush(fp);
	if (whence == SEEK_CUR)
		offset += pos;
	else if (whence == SEEK_END)
		offset += fp->len;
	else if (whence != SEEK_SET)
		offset = -1;
	if (offset < 0) {
		errno = EINVAL;
		return(-1);
	}
	fp->pos = offset;
	fp->eof = 0;
	return(0);
}


int
fsetpos(FILE *fp, const fpos_t *pos)
{
	return(fseek(fp, *pos, SEEK_SET));
}


void
rewind(FILE *fp)
{
	fseek(fp, 0, SEEK_SET);
	fp->err = 0;
}


long
ftell(FILE *fp)
{
	if (fp->refnum <= 0) {
		errno = ENODEV;
		return(-1);
	}
	return(fp->pos - fp->cnt);
}


int
fgetpos(FILE *fp, fpos_t *pos)
{
	if (fp->refnum <= 0) {
		errno = ENODEV;
		return(-1);
	}
	*pos = fp->pos - fp->cnt;
	return(0);
}
