
/*
 *  math.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#define _NOINTRINSICS_
#define _NOSYNONYMS_
#include "math.h"
#include "errno.h"
#include "limits.h"

#if !__option(mc68881)
#include <SANE.h>
#endif


/* ---------- constants ---------- */


	/*  macro for building extended constants  */

#if !__option(native_fp)
	#define _X_(x)		x, x
#elif __option(mc68881)
	#define _X_(x)		x, 0x0000
#else
	#define _X_(x)		x
#endif


	/*  global constants (defined in float.h and math.h)  */

short __F_MIN[]			= { 0x0080, 0x0000 };
short __F_MAX[]			= { 0x7F7F, 0xFFFF };
short __F_EPSILON[]		= { 0x3400, 0x0000 };

#if __option(double_8)
short __D_MIN[]			= { 0x0010, 0x0000, 0x0000, 0x0000 };
short __D_MAX[]			= { 0x7FEF, 0xFFFF, 0xFFFF, 0xFFFF };
short __D_EPSILON[]		= { 0x3CB0, 0x0000, 0x0000, 0x0000 };
short __D_INF[]			= { 0x7FF0, 0x0000, 0x0000, 0x0000 };
#endif

short __X_MIN[]			= { _X_(0x0000), 0x8000, 0x0000, 0x0000, 0x0000 };
short __X_MAX[]			= { _X_(0x7FFE), 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF };
short __X_EPSILON[]		= { _X_(0x3FC0), 0x8000, 0x0000, 0x0000, 0x0000 };
#if !__option(double_8)
short __X_INF[]			= { _X_(0x7FFF), 0x0000, 0x0000, 0x0000, 0x0000 };
#endif


	/*  local constants  */

#if __option(double_8)
static short NegInf[]	= { 0xFFF0, 0x0000, 0x0000, 0x0000 };
#else
static short NegInf[]	= { _X_(0xFFFF), 0x0000, 0x0000, 0x0000, 0x0000 };
#endif

#define Inf			HUGE_VAL
#define NegInf		(* (double *) NegInf)

static double Pi = 3.14159265358979323846;

#if !__option(mc68881)
static short Threshold[] = { _X_(0x3FDE), 0x8000, 0x0000, 0x0000, 0x0000 };
#define Threshold	(* (_double *) Threshold)
static _double Log2Ten = 3.321928094887362348;
#endif


/* ---------- SANE calls ---------- */


#if !__option(mc68881)

#if __option(native_fp)

	#define addr(p)			p
	#define univ(p)			p

#else

	#pragma parameter A0 addr(A0)
	static extended80 *addr(_double *) = {
		0x5488,							//		addq.l	#2,a0
	};
	
	#pragma parameter A0 univ(A0)
	static _double *univ(_double *) = {
		0x30A8, 0x0002					//		move.w	2(a0),(a0)
	};

#endif

#define ABS(p)			(fp68k(addr(p), FABSX), univ(p))
#define ATAN(p)			(elems68k(addr(p), FATANX), univ(p))
#define COS(p)			(elems68k(addr(p), FCOSX), univ(p))
#define EXP(p)			(elems68k(addr(p), FEXPX), univ(p))
#define EXP1(p)			(elems68k(addr(p), FEXP1X), univ(p))
#define LN(p)			(elems68k(addr(p), FLNX), univ(p))
#define LOG2(p)			(elems68k(addr(p), FLOG2X), univ(p))
#define REM(p, q)		(fp68k(addr(q), addr(p), FREMX), univ(p))
#define RINT(p)			(fp68k(addr(p), FRINTX), univ(p))
#define SCALB(p, q)		(fp68k(q, addr(p), FSCALBX), univ(p))
#define SIN(p)			(elems68k(addr(p), FSINX), univ(p))
#define SQRT(p)			(fp68k(addr(p), FSQRTX), univ(p))
#define TAN(p)			(elems68k(addr(p), FTANX), univ(p))
#define TINT(p)			(fp68k(addr(p), FTINTX), univ(p))
#define XPWRY(p, q)		(elems68k(addr(q), addr(p), FXPWRY), univ(p))

#pragma parameter A0 xfersign(A1,A0)
static _double *xfersign(_double *, _double *) = {
	0x7007,								//		moveq	#7,d0
	0x0190,								//		bclr	d0,(a0)
	0x0111,								//		btst	d0,(a1)
	0x6702,								//		beq.s	@1
	0x01D0,								//		bset	d0,(a0)
#if !__option(native_fp)				//	@1
	0x3150,	0x0002,						//		move.w	(a0),2(a0)
#endif
};

#endif


/* ---------- SANE primitives ---------- */


#if !__option(mc68881)

static _double _atan(_double x) { return(*ATAN(&x)); }
static _double _cos(_double x) { return(*COS(&x)); }
static _double _exp(_double x) { return(*EXP(&x)); }
static _double _fabs(_double x) { return(*ABS(&x)); }
static _double _int(_double x) { return(*RINT(&x)); }
static _double _intrz(_double x) { return(*TINT(&x)); }
static _double _log(_double x) { return(*LN(&x)); }
static _double _log10(_double x) { return(*LOG2(&x) / Log2Ten); }
static _double _pow(_double x, _double y) { return(*XPWRY(&x, &y)); }
static _double _sin(_double x) { return(*SIN(&x)); }
static _double _sqrt(_double x) { return(*SQRT(&x)); }
static _double _tan(_double x) { return(*TAN(&x)); }

static _double _acos(_double x)
{
	return(2 * _atan(_sqrt((1 - x) / (1 + x))));
}

static _double _asin(_double x)
{
	_double y = _fabs(x);

	if (y > .5) {
		y = 1 - y;
		y = 2 * y - y * y;
	}
	else
		y = 1 - y * y;
	return(_atan(x / _sqrt(y)));
}

static _double _cosh(_double x)
{
	x = .5 * _exp(_fabs(x));
	return(x + .25 / x);
}

static _double _sinh(_double x)
{
	_double y = _fabs(x);
	
	if (y <= Threshold)
		return(x);
	EXP1(&y);
	y += y / (y + 1);
	y *= .5;
	return(*xfersign(&x, &y));
}

static _double _tanh(_double x)
{
	_double y = _fabs(x);

	if (y <= Threshold)
		return(x);
	y *= -2;
	EXP1(&y);
	y /= y + 2;
	return(*xfersign(&x, &y));
}

static _double _frexp(_double x, int *nptr)
{
	_double y = _fabs(x);
	_double z = 2;
	long n = *LOG2(&y);

	y -= n;
	XPWRY(&z, &y);
	if (z >= 1) {
		z *= .5;
		++n;
	}
	else if (z < .5) {
		z += z;
		--n;
	}
	*nptr = n;
	return(*xfersign(&x, &z));
}

static _double _ldexp(_double x, int n)
{
#if __option(int_4)
	short m = n;
	if (m != n) {
		if (n < 0)
			m = SHRT_MIN;
		else
			m = SHRT_MAX;
	}
	SCALB(&x, &m);
#else
	SCALB(&x, &n);
#endif
	return(x);
}

static _double _fmod(register _double x, _double y)
{
	_double z = x;

	ABS(&y);
	REM(&z, &y);
	if (x > 0 && z < 0)
		z += y;
	else if (x < 0 && z > 0)
		z -= y;
	return(z);
}

#endif


/* ---------- 68881 primitives ---------- */


#if __option(mc68881)

_double _getexp(_double) : 0x1E;
_double _getman(_double) : 0x1F;
_double _intrz(_double) : 0x03;
_double _int(_double) : 0x01;

static _double _fmod(register _double x, _double y)
{
	asm {
		fmod.x	y,x
	}
	return(x);
}

static _double _frexp(_double x, int *nptr)
{
	*nptr = 1 + (int) _getexp(x);
	return(.5 * _getman(x));
}

static _double _ldexp(register _double x, int n)
{
	asm {
#if __option(int_4)
		fscale.l  n,x
#else
		fscale.w  n,x
#endif
	}
	return(x);
}

static _double _pow(_double x, _double y)
{
	return(_exp(y * _log(x)));
}

#endif


/* ---------- error checking ---------- */


#define DomainCheck(test, result)					\
	do {											\
		if (test) {									\
			errno = EDOM;							\
			return(result);							\
		}											\
	} while (0)

#if !__option(mc68881)

	#define Reset()										\
		do {											\
			short env;									\
			fp68k(&env, FPROCENTRY);					\
		} while (0)

	#define RangeCheck(target, result)					\
		do {											\
			short env;									\
			fp68k(&env, FGETENV);						\
			if (env & 0x0D00) {							\
				errno = ERANGE;							\
				target = result;						\
			}											\
		} while (0)

#else

	static void Reset(void) = {
		0x7000,						//		moveq	#0,d0
		0xF200, 0x8800,				//		fmove.l	d0,fpsr
	};
	
	static long _fpsr(void) = {
		0xF200, 0xA800,				//		fmove.l	fpsr,d0
	}; 
	
	#define RangeCheck(target, result)					\
		do {											\
			if (_fpsr() & 0xD0) {						\
				errno = ERANGE;							\
				target = result;						\
			}											\
		} while (0)

#endif


/* ---------- rounding control ---------- */


#if !__option(mc68881)
	
	#define RoundUp()									\
		do {											\
			short env = 0x2000;							\
			fp68k(&env, FSETENV);						\
		} while (0)
	
	#define RoundDown()									\
		do {											\
			short env = 0x4000;							\
			fp68k(&env, FSETENV);						\
		} while (0)
	
	#define RoundNormal()		Reset()

#else

	static void RoundUp(void) = {
		0x7030,						//		moveq	#0x30,d0
		0xF200, 0x9000,				//		fmove.l	d0,fpcr
	};
	
	static void RoundDown(void) = {
		0x7020,						//		moveq	#0x20,d0
		0xF200, 0x9000,				//		fmove.l	d0,fpcr
	};
	
	static void RoundNormal(void) = {
		0x7000,						//		moveq	#0,d0
		0xF200, 0x9000,				//		fmove.l	d0,fpcr
	};

#endif


/* ---------- trigonometric functions ---------- */


/*
 *  acos - inverse circular cosine
 *
 */

double acos(double x)
{
	DomainCheck(x > 1 || x < -1, 0);
	return(_acos(x));
}


/*
 *  asin - inverse circular sine
 *
 */

double asin(double x)
{
	DomainCheck(x > 1 || x < -1, 0);
	return(_asin(x));
}


/*
 *  atan - inverse circular tangent
 *
 */

double atan(double x)
{
	return(_atan(x));
}


/*
 *  atan2 - inverse circular tangent (y/x)
 *
 */

double atan2(double y, double x)
{
	double z;

	DomainCheck(x == 0 && y == 0, 0);
	z = _atan(y / x);
	if (x < 0) {
		if (y < 0)
			z -= Pi;
		else
			z += Pi;
	}
	return(z);
}


/*
 *  cos - circular cosine
 *
 */

double cos(double x)
{	
	return(_cos(x));
}


/*
 *  sin - circular sine
 *
 */

double sin(double x)
{
	return(_sin(x));
}


/*
 *  tan - circular tangent
 *
 */

double tan(double x)
{
	Reset();
	x = _tan(x);
	RangeCheck(x, Inf);
	return(x);
}


/* ---------- hyperbolic functions ---------- */


/*
 *  cosh - hyperbolic cosine
 *
 */

double cosh(double x)
{
	Reset();
	x = _cosh(x);
	RangeCheck(x, Inf);
	return(x);
}


/*
 *  sinh - hyperbolic sine
 *
 */

double sinh(double x)
{
	double y;
	
	Reset();
	y = _sinh(_fabs(x));
	RangeCheck(y, Inf);
	if (x < 0)
		y = -y;
	return(y);
}


/*
 *  tanh - hyperbolic tangent
 *
 */

double tanh(double x)
{
	return(_tanh(x));
}


/* ---------- exponential and logarithmic functions ---------- */


/*
 *  exp - exponential function
 *
 */

double exp(double x)
{
	Reset();
	x = _exp(x);
	RangeCheck(x, Inf);
	return(x);
}


/*
 *  frexp - split floating number into fraction/exponent
 *
 *  This computes a value z, where 0.5 <= fabs(z) < 1.0, and an integer n such
 *  that z*(2^n) == x.
 *
 */

double frexp(double x, int *nptr)
{
	if (x == 0) {
		*nptr = 0;
		return(0);
	}
	return(_frexp(x, nptr));
}


/*
 *  ldexp - combine fraction/exponent into a floating number
 *
 */

double ldexp(double x, int n)
{
	double y;

	Reset();
	y = _ldexp(_fabs(x), n);
	RangeCheck(y, Inf);
	if (x < 0)
		y = -y;
	return(y);
}


/*
 *  log - natural logarithm
 *
 */

double log(double x)
{
	DomainCheck(x < 0, NegInf);
	Reset();
	x = _log(x);
	RangeCheck(x, NegInf);
	return(x);
}


/*
 *  log10 - logarithm base 10
 *
 */

double log10(double x)
{
	DomainCheck(x < 0, NegInf);
	Reset();
	x = _log10(x);
	RangeCheck(x, NegInf);
	return(x);
}


/*
 *  modf - split a floating number into fraction/integer
 *
 */

double modf(double x, double *nptr)
{
	_double y = _intrz(x);

	if (nptr)
		*nptr = y;
	return(x - y);
}


/* ---------- power functions ---------- */


/*
 *  pow - exponentiation
 *
 */

double pow(double x, double y)
{
	int odd = 0;
	
	if (x == 0) {
		DomainCheck(y <= 0, NegInf);
		return(0);
	}
	if (y == 0)
		return(1);
	if (x < 0) {
		DomainCheck(modf(y, 0), NegInf);
		x = -x;
		odd = _fmod(y, 2);
	}
	Reset();
	x = _pow(x, y);
	RangeCheck(x, Inf);
	if (odd)
		x = -x;
	return(x);
}


/*
 *  sqrt - square root
 *
 */

double sqrt(double x)
{
	DomainCheck(x < 0, 0);
	return(_sqrt(x));
}


/* ---------- misc functions ---------- */


/*
 *  ceil - round up to an integer
 *
 */

double ceil(double x)
{
	RoundUp();
	x = _int(x);
	RoundNormal();
	return(x);
}


/*
 *  fabs - absolute value
 *
 */

double fabs(double x)
{
	return(_fabs(x));
}


/*
 *  floor - round down to an integer
 *
 */

double floor(double x)
{
	RoundDown();
	x = _int(x);
	RoundNormal();
	return(x);
}


/*
 *  fmod - remainder function
 *
 *  This computes a value z, with the same sign as x, such that for some
 *  integer k, k*y + z == x.
 *
 */

double fmod(double x, double y)
{
	DomainCheck(y == 0, 0);
	return(_fmod(x, y));
}
