/******************************************************************************
 OFile.c

							The File Class
		
	Class for a disk-based file. Information can be read from or written to
	a File.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "OFile.h"
#include "TBUtilities.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IFile

		Initialize a File object
 ******************************************************************************/

void	OFile::IFile()
{
	name[0] = 0;						/* Clear instance variables			*/
	volNum = 0;
	dirID = 0L;
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a File object
 ******************************************************************************/

void	OFile::Dispose()
{
	Close();							/* Close Mac File					*/
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Open

		Open a File. Subclasses must override this method.
 ******************************************************************************/

OSErr	OFile::Open(
	SignedByte		permission)
{
}										/* Null Method						*/


/******************************************************************************
 Close

		Close a File. Subclasses must override this method.
 ******************************************************************************/

OSErr	OFile::Close()
{
}										/* Null Method						*/


/**** S P E C I F Y I N G   M E T H O D S ****/


/******************************************************************************
 Specify

		Specify the name and volume number of a File
 ******************************************************************************/

void	OFile::Specify(
	Str63		aName,
	short		aVolNum)
{
	CopyPString(aName, name);
	volNum = aVolNum;
	dirID = 0L;
}


/******************************************************************************
 SpecifyHFS

		Specify the name, volume number, and directory ID of a File
 ******************************************************************************/

void	OFile::SpecifyHFS(
	Str63		aName,
	short		aVolNum,
	long		aDirID)
{
	CopyPString(aName, name);
	volNum = aVolNum;
	dirID = aDirID;
}


/******************************************************************************
 SFSpecify

		Specify file parameters from a Standard File reply record
 ******************************************************************************/

void	OFile::SFSpecify(
	SFReply		*macSFReply)
{
	short		aVolNum;
	long		aDirID;
	long		aProcID;
													
	GetWDInfo(macSFReply->vRefNum, &aVolNum, &aDirID, &aProcID);
	SpecifyHFS(macSFReply->fName, aVolNum, aDirID);
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 GetName

		Retrieve the name of a File
 ******************************************************************************/

void	OFile::GetName(
	Str63		theName)
{
	CopyPString(name, theName);
}


/**** F I L I N G   M E T H O D S ****/


/******************************************************************************
 CreateNew

		Create a new file
 ******************************************************************************/

OSErr	OFile::CreateNew(
	OSType		creator,				/* Creator signature				*/
	OSType		fType)					/* File type						*/
{
	return( HCreate(volNum, dirID, name, creator, fType) );
}


/******************************************************************************
 ThrowOut

		Remove a file from disk, i.e., throw it in the trash.
 ******************************************************************************/

OSErr	OFile::ThrowOut()
{
	Close();							/* Must close file before we can	*/
										/*   throw it out					*/
	return( HDelete(volNum, dirID, name) );
}


/******************************************************************************
 ChangeName

		Rename a file. File may or may not be open.
 ******************************************************************************/

OSErr	OFile::ChangeName(
	Str63		newName)
{
	OSErr		errCode;
	
	errCode = HRename(volNum, dirID, name, newName);
	
	if (errCode == noErr) {
		CopyPString(newName, name);
	}
	return(errCode);
}