/******************************************************************************
 CBartender.h

		Interface for the Bartender Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- added EnableMenuBar and DisableMenuBar methods
		- added UpdateMenuBar method
		- added inMenuBar flag to MenuEntry, to indicate whether the
		  menu is currently installed in the menu bar.
		- added lastEnable flag to MenuEntry, to indicate the last
		  seen enable state for a menu.
		- added forceMBarUpdate instance variable.
	    - merged CBartender with CBarOwner. CBartender gained a choreAssigned
	      instance variable.
	]

 ******************************************************************************/
 
#define _H_CBartender

#include "CObject.h"					/* Interface for its superclass		*/

typedef enum {
	dimNONE,
	dimSOME,
	dimALL
} DimOption;

typedef struct	{
	short			MENUid;
	MenuHandle		macMenu;
	DimOption		dimming;
	Boolean			unchecking	: 1;
	Boolean			hasHMenus	: 1;
	Boolean			inMenuBar	: 1;
	Boolean			lastEnable  : 1;
	short			numCmds;
	long			**theCommands;
} MenuEntry, *MenuEntryP, **MenuEntryH;

class CBartender : public CObject {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	short			numMenus;			/* Number of menus					*/
	MenuEntryH		theMenus;			/* Table of available menus			*/
	Boolean		choreAssigned;			/* Has menu bar chore been created?	*/
	Boolean		forceMBarUpdate;		/* if TRUE UpdateMenuBar always draws */

	
								/** Instance Methods **/
									/** Construction **/
	void		IBartender(short MBARid);
	
									/** Menu Insertion / Deletion **/
	virtual void		AddMenu(short MENUid, Boolean install, short beforeID);
	virtual void		RemoveMenu(short MENUid);
	virtual void		InsertInBar(short MENUid, short beforeID);
	virtual void		DeleteFromBar(short MENUid);
	virtual void		InsertHierMenu(short hMENUid, long cmdNo,
								short inMENUid, short afterItem);
	
									/** Item Manipulation **/
	virtual void		EnableCmd(long cmdNo);
	virtual void		DisableCmd(long cmdNo);
	virtual void		EnableMenu(short MENUid);
	virtual void		DisableMenu(short MENUid);
	virtual void		EnableMenuBar( void);
	virtual void		DisableMenuBar( void);
	virtual void		SetCmdText(long cmdNo, Str255 theText);
	virtual void		GetCmdText(long cmdNo, Str255 theText);
	virtual void		CheckMarkCmd(long cmdNo, Boolean checked);
	
									/** Item Insertion / Deletion **/
	virtual void		InsertMenuCmd(long cmdNo, Str255 theText,
								short MENUid, short afterItem);
	virtual void		RemoveMenuCmd(long cmdNo);

									/** Look-up **/
	virtual short		FindMenuIndex(short MENUid);
	virtual MenuHandle	FindMacMenu(short MENUid);
	virtual long		FindCmdNumber(short MENUid, short itemNo);
	virtual void		FindMenuItem(long cmdNo, short *MENUid, MenuHandle *macMenu,
								short *itemNo);
	virtual short		FindItemText(short MENUid, Str255 itemStr);
									
									/** Command Extraction **/
	virtual void		ExtractCommands(MenuEntryP theEntry);
	virtual void		ParseItemString(Str255 itemStr, long *cmdNo);
	virtual void		ExtractHierMenus(MenuHandle macMenu, short index);
	
									/** Appearance **/
	virtual void		SetDimOption(short MENUid, DimOption aDimming);
	virtual void		SetUnchecking(short MENUid, Boolean anUnchecking);
	virtual void		UpdateAllMenus(void);
	virtual void		UpdateMenuBar( void);
};

	/** Macros **/
	
										/* Check whether an entire menu is	*/
										/*   enabled						*/
#define		MenuEnabled(macMenu)	((**macMenu).enableFlags & 1L)