/******************************************************************************
 CPrinter.h

		Interface for the Printer Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added printMgrOprn, printDocOpen, printPageOpen, savedResFile,
		  printDirection instance variables
		- changed interface for OpenPrintMgr
		- added ClosePrintMgr, SetPrintDir, HavePagination, ResetPagination,
		  SetStrips, SetHorizPageBreak, SetVertPageBreak, SetAllStripWidths,
		  SetStripHeight, SetAllStripHeights, GetStripCount, PageNumToStrips,
		  GetPageStart, GetPageArea methods
	] 

 ******************************************************************************/
 
#define _H_CPrinter

#include "CObject.h"					/* Interface for its superclass		*/
#include <PrintTraps.h>

	/* parameters for SetStripWidths, SetStripHeights to use paper 			*/
	/* printable width or height											*/
	
#define		kUsePaperWidth		-1
#define		kUsePaperHeight		-1

typedef enum
{
	printHoriz,		/* printing proceeds row by row 		*/
	printVert		/* printing proceeds column by column 	*/
	
}	tPrintDirection;

	// Forward declarations
	
class CRunArray;
class CDocument;
struct LongPt;
struct LongRect;

class CPrinter : public CObject {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	CDocument		*itsDocument;		/* Document using this Printer		*/
	THPrint			macTPrint;			/* Toolbox print record				*/
	TPPrPort		macPrintPort;
	tPrintDirection printDirection;
	Boolean			printMgrOpen;
	Boolean			printDocOpen;
	Boolean			printPageOpen;
	short			savedResFile;
	
	struct CRunArray *itsStripWidths;
	struct CRunArray *itsStripHeights;

								/** Instance Methods **/
									/** Contruction/Destruction **/
	Boolean		IPrinter(CDocument *aDocument, THPrint aMacTPrint);
	virtual void		Dispose(void);
	
									/** Accessing **/
	virtual Boolean		OpenPrintMgr( Boolean fCheckFailure);
	virtual void		ClosePrintMgr( void);
	virtual THPrint		GetPrintRecord(void);
	virtual void		GetPageInfo(Rect *paperRect, Rect *pageRect,
								short *hRes, short *vRes);
	virtual Boolean		DoPageSetup(void);
	
	virtual void		SetPrintDir( tPrintDirection aPrintDir);
	virtual Boolean		HavePagination( void);
	virtual void		ResetPagination( void);
	virtual void		SetStrips( short numHStrips, short numVStrips);
	
	virtual void		SetHorizPageBreak( short vStripNum, long hPos);
	virtual void		SetVertPageBreak( short hStripNum, long vPos);
	virtual void		SetAllStripWidths( short aStripWidth);
	virtual void		SetStripWidth( short pageNum, short aStripWidth);
	virtual void		SetAllStripHeights( short aStripHeight);
	virtual void		SetStripHeight( short pageNum, short aStripHeight);
	
	virtual void		GetStripCount( short *hStrips, short *vStrips);
	
	virtual void		PageNumToStrips( short pageNum, short *hStrip, short *vStrip);
	virtual void		GetPageStart( short pageNum, struct LongPt *startPos);
	virtual void		GetPageArea( short pageNum, struct LongRect *pageArea);
	
									/** Printing **/
	virtual void		DoPrint(void);
	virtual void		PrintPageRange(short firstPage, short lastPage);
};