/******************************************************************************
 CRunArray.h

		Interface for RunArray class
		
	SUPERCLASS = CArray
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#define _H_CRunArray

#include "CArray.h"

	/* structure used to store run info	*/
	
typedef struct tRun
{
	long	runLength;		/* number of consecutive entries with same value	*/
	long	value;			/* the value										*/
	
} tRun, *tRunPtr, **tRunHndl;


class CRunArray : public CArray
{
public:
		
	void	IRunArray( void);
	
	virtual void	InsertValue( long item, long value, long count);
	virtual void	SetValue( long index, long value);
	virtual long	GetValue( long index);
	virtual void	DeleteValue( long index);
	virtual void	DeleteAll( void);
	
	virtual long	SumRange( long startIndex, long endIndex);
	virtual long	FindSum( long aSum);
	
	virtual long	GetNumItems( void);
	
	virtual CObject* Copy(void);
	
protected:
	
	long		itemCount;	/* number of values in array - numItems is number of runs	*/
	tRunHndl	hRuns;		/* handle to runs, same as CArray::hItems					*/

	virtual void	FindRun( long itemIndex, long *runIndex, long *firstInRun);
	virtual void	InsertRun( long index, long runLength, long value);
	virtual void	DeleteRun( long runIndex);

};

/* change protocol	*/

	/*
	 *	CRunArray will send its dependents the runArraySizeChanged
	 *  change message when InsertValue, DeleteValue, or DeleteAll
	 *  methods are called. Since it is a subclass of CArray, it 
	 *  may also send CArray change messages, but those signal
	 *  changes in the runs, not in the elements themselves, and
	 *  will not usually be of interest to dependents.
	 */
	 
enum
{							/* info parameter is pointer to new size	*/
	runArraySizeChanged = arrayLastChange + 1,
	
	runArrayLastChange = runArraySizeChanged
	
};