/******************************************************************************
 Global.h
 
		Header file for commonly used constants, type definitions, and macros
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added ForgetObject, ForgetHandle and ForgetPtr macros
		- changed NULL macro for ANSI compatability
		- added tSystem declaration
	]

 ******************************************************************************/

#define _H_Global


#ifndef NULL
#define		NULL		((void *) 0)	/* Nil pointer value				*/
#endif

#define		MAXINT		32767			/* Maximum int value				*/
#define		MININT		-32768			/* Mininum int value				*/
#define		MAXLONG		0x7FFFFFFF		/* Maximum long int value			*/
#define		MINLONG		0x80000000		/* Minimum long int value			*/
#define		NOTHING		0				/* Useful flag						*/

										/* For convenience, define types	*/
										/*   for handles to:				*/
typedef		short		**ShortHandle;	/*     a short integer				*/
typedef		long		**LongHandle;	/*     a long integer				*/

typedef		void (*VoidFunc)(...);			/* Ptr to a function returning void	*/
typedef		Boolean (*BooleanFunc)(...);	/* Ptr to a Boolean function		*/
typedef		long (*LongFunc)(...);			/* Ptr to a function returning long	*/

										/* Efficient C versions of HiWord	*/
										/*   and LoWord Traps				*/
#define		HiShort(longNum)	(short)(((longNum) >> 16) & 0xFFFF)
#define		LoShort(longNum)	(short)((longNum) & 0xFFFF)

										/* Macros for extracting the first	*/
										/*   and last 8 bits of a short		*/
#define		HiByte(shortNum)	(((shortNum) >> 8) & 0x00FF)
#define		LoByte(shortNum)	((shortNum) & 0x00FF)

										/* Macro for absolute value			*/
										/*   x may be any numeric type		*/
#define		Abs(x)			((x) < 0 ? -(x) : (x))

										/* Macros for maximum and minimum	*/
										/*   of two numbers					*/
#define		Max(x, y)		((x) > (y) ? (x) : (y))
#define		Min(x, y)		((x) < (y) ? (x) : (y))

										/* turn on QuickDraw's hilite mode	*/
						
void SetHiliteMode(void) = {0x08B8, 0x0007, 0x0938}; // bclr #7,0x938

/*
 *	The following macros are handy for disposing of objects,
 *	handles, pointers, and resources in a safe manner
 *  The macros expand to calls to the following functions,
 *	defined in TCLUtilities.c
 */
pascal void		__ForgetHandle( Handle *handleAddr);
pascal void		__ForgetPtr( Ptr *ptrAddr);
pascal void		__ForgetObject( Ptr *objAddr);
pascal void		__ForgetResource( Handle *rsrcAddr);

#define ForgetHandle(h) 	__ForgetHandle( (Handle*) &h)
#define ForgetPtr(p) 		__ForgetPtr( (Ptr*) &p)
#define ForgetResource(h)	__ForgetResource( (Handle*) &h)
#define ForgetObject(obj) 	__ForgetObject( (Ptr*) &obj)

typedef struct
{
	Boolean		hasWNE 			: 1;
	Boolean		hasColorQD		: 1;
	Boolean		hasGestalt		: 1;
	Boolean		hasAppleEvents	: 1;
	Boolean		hasAliasMgr		: 1;
	Boolean		hasEditionMgr	: 1;
	Boolean		hasHelpMgr		: 1;
	Boolean		hasScriptMgr	: 1;
	Boolean		hasFPU			: 1;
	short		scriptsInstalled;
	short		systemVersion;
	
} tSystem;

extern tSystem gSystem;
