/******************************************************************************
 CSizeBox.c

							The SizeBox Class
		
		The size box (or grow box) which sits in the lower right hand
		corner of resizeable windows.
	
	SUPERCLASS = CPane

	Copyright  1989 Symantec Corporation. All rights reserved.
	
 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "CSizeBox.h"
#include "TBUtilities.h"


/******************************************************************************
 ISizeBox

		Initialize a SizeBox object. A SizeBox is automatically located in
		the bottom right corner of its enclosure.
 ******************************************************************************/

void	CSizeBox::ISizeBox(
	CView		*anEnclosure,
	CBureaucrat	*aSupervisor)
{
	LongRect		enclFrame;
	
	useSICN = FALSE;
	
	anEnclosure->GetFrame(&enclFrame);
	
	CPane::IPane(anEnclosure, aSupervisor, SIZEBOX_LENGTH, SIZEBOX_LENGTH,
					enclFrame.right - SIZEBOX_LENGTH,
					enclFrame.bottom - SIZEBOX_LENGTH,
					sizFIXEDRIGHT, sizFIXEDBOTTOM);
	wantsClicks = TRUE;
}
	

/******************************************************************************
 Draw {OVERRIDE}

		Draw a SizeBox
 ******************************************************************************/

void	CSizeBox::Draw(
	Rect		*area)
{
	Rect		theFrame;
	Point		pt;

	if (IsActive()) 
	{					/* Use SICN Resource for active box	*/
		if (useSICN)
		{
			LongToQDPt( &topLeftL( frame), &pt);
			DrawSICN(SICN_SIZEBOX, 1, pt);
		}
		else
		{
			PenNormal();
			DrawGrowIcon( (WindowPtr) macPort);
		}
		
	} else {							/* Inactive size box is blank		*/
		LongToQDRect( &frame, &theFrame);
		FrameRect(&theFrame);
		InsetRect(&theFrame, 1, 1);
		EraseRect(&theFrame);
	}
}
	

/******************************************************************************
 Activate {OVERRIDE}

		Indicate an active SizeBox
 ******************************************************************************/

void	CSizeBox::Activate()
{
	Rect		tempRect;
	
	if (!active) {						/* If size box is not active ...	*/
		active = TRUE;					/* Set flag							*/
		if (ReallyVisible()) {			/* Make sure it's on screen			*/
			FrameToWindR( &aperture, &tempRect);
										/* Force redraw so that an active	*/
			DrawAll(&tempRect);			/*   size box is displayed			*/
			ValidRect(&tempRect);		/* Ensure that it's not drawn again	*/
		}
	}
}
	

/******************************************************************************
 Deactivate {OVERRIDE}

		Indicate an inactive SizeBox
 ******************************************************************************/

void	CSizeBox::Deactivate()
{
	Rect		tempRect;
	
	if (active) {						/* If size box is active			*/
		active = FALSE;					/* Clear flag						*/
		if (ReallyVisible()) {			/* Make sure it's on screen			*/
			FrameToWindR( &aperture, &tempRect);
										/* Force redraw so that an inactive	*/
			DrawAll(&tempRect);			/*   size box is displayed			*/
		}
	}
}
