/******************************************************************************
 GenericMDEF.c

		
	A generic Menu Definition Procedure. Extracts the reference to a
	CMenuDefProc object from the MDEF handle and sends messages to that
	object for performing the actions requested by the Toolbox. This
	defProc routine is called by the Toolbox Menu Manager when some
	action needs to be taken for a custom menu.
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
 ******************************************************************************/

#include "Global.h"
#include "GenericMDEF.h"


/*=============================================================================
 GenericMDEF
 
		A Generic Menu Definition Procedure which uses Object C. 
 =============================================================================*/

pascal void	GenericMDEF(
	short					theMessage,	/* Type of action required			*/
	register MenuHandle		macMenu,	/* The menu to manipulate			*/
	Rect					*menuRect,	/* Bounding box of the menu			*/
	Point					hitPt,		/* Global coords of mouse click		*/
	short					*whichItem)	/* On entry: previous selection		*/
										/*  On exit: new selection			*/
{
	GenericMDEFHand		theMDEF;
	CMenuDefProc		*theMenuDefProc;
	
	theMDEF = (GenericMDEFHand) (**macMenu).menuProc;
	theMenuDefProc = (**theMDEF).itsMenuDefProc;
	
	switch (theMessage) {				/* Take appropriate action			*/
	
		case mDrawMsg:					/* Draw the menu					*/
			theMenuDefProc->DrawMenu(macMenu, menuRect);
			break;
			
		case mChooseMsg:				/* Handle selection from the menu	*/
			theMenuDefProc->ChooseItem(macMenu, menuRect, hitPt, whichItem);
			break;
			
		case mSizeMsg:					/* Determine dimensions of the menu	*/
			theMenuDefProc->SizeMenu(macMenu);
			break;
		
		case mPopUpMsg:					/* Specify placement of popup menu	*/
			theMenuDefProc->PlacePopup(macMenu, menuRect, hitPt, whichItem);
			break;
	}
}
