/******************************************************************************
 CStack.c

							The Stack Class
		
		A push-down stack data structure for storing objects in a Last In,
		First Our (LIFO) manner.
	
	SUPERCLASS = CCluster
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CStack.h"
	

/******************************************************************************
 IStack

		Initialize a Stack object
 ******************************************************************************/

void	CStack::IStack()
{
	CCluster::ICluster();				/* Initialize superclass			*/
}
	

/******************************************************************************
 Push

		Push an object on the stack
 ******************************************************************************/

void	CStack::Push(
	CObject		*theObject)				/* Object to push on the stack		*/
{
	CCluster::Add(theObject);			/* Same as adding to a cluster		*/
}
	

/******************************************************************************
 Pop

		Pop an object off the stack. Returns a pointer to the
		popped object.
 ******************************************************************************/

CObject*	CStack::Pop()
{
	CObject*	poppedItem = NULL;
	
	if (numItems > 0) {
		numItems--;
		poppedItem = (CObject*) (*items)[numItems];
		(*items)[numItems] = 0;
		
		if (slots > numItems + blockSize) {
										/* The number of free slots is		*/
										/*   greater than the blockSize.	*/
										/*   Reduce the size of the items	*/
										/*   handle.						*/
			Resize( slots-blockSize);
		}
	}
	return(poppedItem);
}
