/*
 *	Exceptions.h
 *
 *	Copyright  1991 Symantec Corporation. All rights reserved.
 *
 *	Based on material copyrighted by Apple Computer, Inc., 1985-1988.
 *
 */
	

#pragma once

#include <setjmp.h>

#define kSilentErr				1	// pass to Failure to fail without any user alert

typedef struct FailInfo
{
	long			A6Link;
	char			handlerType;
	char			fPropagate;
	void			*handlerProc;
	struct FailInfo	*next;
	jmp_buf			regs;
	
} FailInfo;


typedef pascal void (*HandlerFuncPtr)( short error, long message);
 	
extern char			gAskFailure;	/* When TRUE, break into debugger on functions
									 *	that may conditionally fail, e.g. FailNIL().
									 *	Provides opportunity for simulating failures
									 */
										
extern char 		gBreakFailure;	/* When TRUE, break into debugger when a failure
									 *	has occurred.
									 */

extern short		gLastError;		/* last error code that caused a failure  */
extern long			gLastMessage;	/* last message associated with a failure */



void PushTryHandler( FailInfo *fi);
pascal short CatchException( FailInfo *fi);
pascal void CatchFailures(FailInfo *fi, HandlerFuncPtr handler, long staticLink);
pascal void Failure(short error, long message);
pascal void FailMemError( void);
pascal void FailResError( void);
pascal void FailNIL( void *p);
pascal void FailOSErr( short error);
pascal void FailNILRes( void* p);
pascal void	Success(void);
pascal void RetryException( FailInfo *fi);

pascal long SpecifyMsg( short strListID, short strIndex);
pascal void SetFailInfo( short newError, long newMessage);

#pragma nooptimize(CatchException)
				


#define TRY									\
		{ FailInfo __fi;					\
		  PushTryHandler( &__fi);			\
		  if (!setjmp( __fi.regs)) {
		
#define CATCH								\
			Success(); }					\
		else {
		
#define ENDTRY								\
		if (__fi.fPropagate)				\
		Failure( gLastError, gLastMessage);	\
		}	}
			
#define RETRY						\
		RetryException( &__fi)
		
#define NO_PROPAGATE				\
		__fi.fPropagate = 0
		
/***	Exception strings defined by the TCL	***/

enum
{
	excMsgNullObject = 1,
	excMsgLookupFailed,
	excNewByNameFailed,
	excPrintJobTooBig,
	excExceedTELimit,
	
	excLastTCLException = excExceedTELimit
};


#undef ASSERT

#ifdef __TCL_DEBUG__

#define ASSERT( condition)	\
	condition ? 0 : __assertion_failed( #condition, __FILE__, __LINE__)

short __assertion_failed( char *conditionStr,  char *fileName, int lineNum);

#else

#define ASSERT( condition)		(0)

#endif
